@@ -1,6 +1,6 @@
 /*
  * MOC - music on console
- * Copyright (C) 2004-2005 Damian Pietras <daper@daper.net>
+ * Copyright (C) 2004-2006 Damian Pietras <daper@daper.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #define DEBUG
 
 #include "server.h"
-#include "main.h"
+#include "common.h"
 #include "decoder.h"
 #include "playlist.h"
 #include "log.h"
@@ -94,6 +94,10 @@ static struct sound_params req_sound_params = { 0, 0, 0 };
 static struct audio_conversion sound_conv;
 static int need_audio_conversion = 0;
 
+/* URL of the last played stream. Used to fake pause/unpause of internet
+ * streams. Protected by curr_playing_mu. */
+static char *last_stream_url = NULL;
+
 /* Check if the two sample rates don't differ as much that we can't play. */
 #define sample_rate_compat(sound, device) ((device) * 1.05 >= sound \
 		&& (device) * 0.95 <= sound)
@@ -226,7 +230,7 @@ static long sfmt_best_matching (const long formats_with_endian,
 	assert (best != 0);
 
 	if (!(best & (SFMT_S8 | SFMT_U8)))
-		best |= formats_with_endian & SFMT_MASK_ENDIANES;
+		best |= req_with_endian & SFMT_MASK_ENDIANES;
 
 	debug ("Choosed %s as the best matching %s",
 			sfmt_str(best, fmt_name1, sizeof(fmt_name1)),
@@ -286,24 +290,48 @@ static void go_to_another_file ()
 	 * by searching for the current item on the list where we are
 	 * switching. */
 	if (shuffle && curr_plist != &shuffled_plist) {
-		curr_playing = plist_find_del_fname (&shuffled_plist,
-				plist_get_file(&playlist, curr_playing));
-		assert (curr_playing != -1);
+		char *file = plist_get_file (&playlist, curr_playing);
+
+		if (file) {
+			curr_playing = plist_find_del_fname (&shuffled_plist,
+					file);
+			free (file);
+		}
 	}
 	else if (!shuffle && curr_plist != &playlist) {
-		curr_playing = plist_find_del_fname (&playlist,
-				plist_get_file(&shuffled_plist, curr_playing));
-		assert (curr_playing != -1);
+		char *file = plist_get_file (&shuffled_plist, curr_playing);
+
+		if (file) {
+			curr_playing = plist_find_del_fname (&playlist,
+					file);
+			free (file);
+		}
 	}
 	
 	if (shuffle)
 		curr_plist = &shuffled_plist;
 	else
 		curr_plist = &playlist;
+
+	/* If curr_playing == -1 now, the playlist is empty.
+	 * 
+	 * Order of files on the playlist could have been changed, so we can't
+	 * just use plist_prev()/plist_next(), we must first find the current
+	 * position of the last played file. */
 	
-	if (play_prev == 1) { 
+	if (play_prev == 1 && curr_playing != -1) { 
+		int curr_playing_current_pos;
+
 		logit ("Playing previous...");
-		curr_playing = plist_prev (curr_plist, curr_playing);
+		
+		curr_playing_current_pos = plist_find_fname (curr_plist,
+				curr_playing_fname);
+
+		if (curr_playing_current_pos != -1)
+			curr_playing = plist_prev (curr_plist,
+					curr_playing_current_pos);
+		else
+			curr_playing = plist_prev (curr_plist, curr_playing);
 		if (curr_playing == -1) {
 			if (options_get_int("Repeat"))
 				curr_playing = plist_last (curr_plist);
@@ -312,8 +340,18 @@ static void go_to_another_file ()
 		else 
 			logit ("Previous item.");
 	}
-	else if (options_get_int("AutoNext") || play_next) {
-		curr_playing = plist_next (curr_plist, curr_playing);
+	else if (curr_playing != -1
+			&& (options_get_int("AutoNext") || play_next)) {
+		int curr_playing_current_pos;
+
+		curr_playing_current_pos = plist_find_fname (curr_plist,
+				curr_playing_fname);
+
+		if (curr_playing_current_pos != -1)
+			curr_playing = plist_next (curr_plist,
+					curr_playing_current_pos);
+		else
+			curr_playing = plist_next (curr_plist, curr_playing);
 		if (curr_playing == -1 && options_get_int("Repeat")) {
 			if (shuffle) {
 				plist_clear (&shuffled_plist);
@@ -384,9 +422,9 @@ static void *play_thread (void *unused ATTR_UNUSED)
 		}
 
 		LOCK (curr_playing_mut);
-		if (curr_playing_fname) {
-			free (curr_playing_fname);
-			curr_playing_fname = NULL;
+		if (last_stream_url) {
+			free (last_stream_url);
+			last_stream_url = NULL;
 		}
 		UNLOCK (curr_playing_mut);
 
@@ -402,6 +440,11 @@ static void *play_thread (void *unused ATTR_UNUSED)
 		state_change ();
 	}
 
+	if (curr_playing_fname) {
+		free (curr_playing_fname);
+		curr_playing_fname = NULL;
+	}
+
 	audio_close ();
 	logit ("exiting");
 
@@ -430,6 +473,17 @@ void audio_stop ()
 		stop_playing = 0;
 		logit ("done stopping");
 	}
+	else if (state == STATE_PAUSE) {
+		
+		/* Paused internet stream - we are in fact stopped already. */
+		if (curr_playing_fname) {
+			free (curr_playing_fname);
+			curr_playing_fname = NULL;
+		}
+
+		state = STATE_STOP;
+		state_change ();
+	}
 }
 
 /* Start playing from the file fname. If fname is an empty string,
@@ -468,14 +522,11 @@ void audio_play (const char *fname)
 			curr_playing = -1;
 	}
 	
-	if (curr_playing != -1) {
-		if (pthread_create(&playing_thread, NULL, play_thread, NULL))
-			error ("can't create thread");
-		play_thread_running = 1;
-	}
-	else
-		logit ("Client wanted to play a file not present on the "
-				"playlist.");
+	if (pthread_create(&playing_thread, NULL, play_thread,
+				curr_playing != -1 ? NULL : (void *)fname))
+		error ("can't create thread");
+	play_thread_running = 1;
+	
 	UNLOCK (plist_mut);
 	UNLOCK (curr_playing_mut);
 }
@@ -504,11 +555,26 @@ void audio_pause ()
 	if (curr_playing != -1) {
 		char *sname = plist_get_file (curr_plist, curr_playing);
 		
-		if (file_type(sname) != F_URL) {
-			out_buf_pause (&out_buf);
-			state = STATE_PAUSE;
-			state_change ();
+	
+		if (file_type(sname) == F_URL) {
+			UNLOCK (curr_playing_mut);
+			UNLOCK (plist_mut);
+			audio_stop ();
+			LOCK (curr_playing_mut);
+			LOCK (plist_mut);
+
+			if (last_stream_url)
+				free (last_stream_url);
+			last_stream_url = xstrdup (sname);
+
+			/* Pretend that we are paused on this. */
+			curr_playing_fname = xstrdup (sname);
 		}
+		else
+			out_buf_pause (&out_buf);
+		
+		state = STATE_PAUSE;
+		state_change ();
 		
 		free (sname);
 	}
@@ -519,9 +585,22 @@ void audio_pause ()
 
 void audio_unpause ()
 {
-	out_buf_unpause (&out_buf);
-	state = STATE_PLAY;
-	state_change ();
+	LOCK (curr_playing_mut);
+	if (last_stream_url && file_type(last_stream_url) == F_URL) {
+		char *url = xstrdup (last_stream_url);
+
+		UNLOCK (curr_playing_mut);
+		audio_play (url);
+		free (url);
+	}
+	else if (curr_playing != -1) {
+		out_buf_unpause (&out_buf);
+		state = STATE_PLAY;
+		state_change ();
+		UNLOCK (curr_playing_mut);
+	}
+	else
+		UNLOCK (curr_playing_mut);
 }
 
 static void reset_sound_params (struct sound_params *params)
@@ -764,7 +843,7 @@ static void print_output_capabilities (const struct output_driver_caps *caps)
 			sfmt_str(caps->formats, fmt_name, sizeof(fmt_name)));
 }
 
-void audio_init ()
+void audio_initialize ()
 {
 	find_working_driver (options_get_str("SoundDriver"), &hw);
 
@@ -794,6 +873,9 @@ void audio_exit ()
 		logit ("Can't destroy plist_mut: %s", strerror(errno));
 	if (pthread_mutex_destroy(&request_mut))
 		logit ("Can't destroy request_mut: %s", strerror(errno));
+
+	if (last_stream_url)
+		free (last_stream_url);
 }
 
 void audio_seek (const int sec)
@@ -819,16 +901,16 @@ void audio_plist_add (const char *file)
 {
 	LOCK (plist_mut);
 	plist_clear (&shuffled_plist);
-	plist_add (&playlist, file);
+	if (plist_find_fname(&playlist, file) == -1)
+		plist_add (&playlist, file);
+	else
+		logit ("Wanted to add a file that is already present on the "
+				"list: %s", file);
 	UNLOCK (plist_mut);
 }
 
 void audio_plist_clear ()
 {
-	/* We must stop before clearing the playlist, because w playing thread
-	 * is accessing the playlist items. */
-	audio_stop ();
-	
 	LOCK (plist_mut);
 	plist_clear (&shuffled_plist);
 	plist_clear (&playlist);
@@ -905,7 +987,7 @@ void audio_plist_set_time (const char *file, const int time)
 
 	LOCK (plist_mut);
 	if ((i = plist_find_fname(&playlist, file)) != -1) {
-		update_item_time (&playlist.items[i], time);
+		plist_set_item_time (&playlist, i, time);
 		playlist.items[i].mtime = get_mtime (file);
 		debug ("Setting time for %s", file);
 	}
@@ -940,6 +1022,14 @@ void audio_plist_set_serial (const int serial)
 	UNLOCK (plist_mut);
 }
 
+/* Swap 2 file on the playlist. */
+void audio_plist_move (const char *file1, const char *file2)
+{
+	LOCK (plist_mut);
+	plist_swap_files (&playlist, file1, file2);
+	UNLOCK (plist_mut);
+}
+
 struct file_tags *audio_get_curr_tags ()
 {
 	return player_get_curr_tags ();
