@@ -2,9 +2,8 @@
 #define INTERFACE_H
 
 void init_interface (const int sock, const int logging, char **args,
-		const int arg_num, const int recursively);
+		const int arg_num);
 void interface_loop ();
-int server_connect ();
 void interface_end ();
 int user_wants_interrupt ();
 void interface_error (const char *msg);
@@ -22,4 +21,6 @@ void interface_cmdline_append (int server_sock, char **args,
 void interface_cmdline_play_first (int server_sock);
 void interface_cmdline_file_info (const int server_sock);
 void interface_cmdline_playit (int server_sock, char **args, const int arg_num);
+void interface_cmdline_seek_by (int server_sock, const int seek_by);
+
 #endif
