@@ -34,7 +34,7 @@
 
 #define DEBUG
 
-#include "main.h"
+#include "common.h"
 #include "log.h"
 #include "io.h"
 #include "options.h"
@@ -688,7 +688,7 @@ char *io_strerror (struct io_stream *s)
 		s->strerror = xstrdup (err);
 	}
 	else
-		s->strerror = strdup ("OK");
+		s->strerror = xstrdup ("OK");
 
 	return s->strerror;
 }
