@@ -1,6 +1,6 @@
 /*
  * MOC - music on console
- * Copyright (C) 2004-2005 Damian Pietras <daper@daper.net>
+ * Copyright (C) 2004 - 2006 Damian Pietras <daper@daper.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,9 @@
 #include <strings.h>
 #include <assert.h>
 
-#ifdef HAVE_NCURSES_H
+#ifdef HAVE_NCURSESW_H
+# include <ncursesw/curses.h>
+#elif HAVE_NCURSES_H
 # include <ncurses.h>
 #elif HAVE_CURSES_H
 # include <curses.h>
@@ -32,7 +34,7 @@
 #include "interface.h"
 #include "options.h"
 #include "log.h"
-#include "main.h"
+#include "common.h"
 #include "files.h"
 
 #define CTRL_KEY_CODE	0x1F
@@ -191,10 +193,18 @@ static struct command commands[] = {
 		1
 	},
 	{
-		KEY_CMD_TOGGLE_PLAYLIST,
-		"toggle_playlist",
+		KEY_CMD_TOGGLE_MENU,
+		"toggle_menu",
 		"Switch between playlist and file list",
 		CON_MENU,
+		{ '\t', -1 },
+		1
+	},
+	{
+		KEY_CMD_TOGGLE_LAYOUT,
+		"toggle_layout",
+		"Switch between layouts",
+		CON_MENU,
 		{ 'l', -1 },
 		1
 	},
@@ -597,6 +607,118 @@ static struct command commands[] = {
  		CON_MENU,
  		{ 'x', -1 },
  		1
+  	},
+ 	{
+ 		KEY_CMD_PLIST_MOVE_UP,
+ 		"plist_move_up",
+ 		"Move playlist item up",
+ 		CON_MENU,
+ 		{ 'u', -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_PLIST_MOVE_DOWN,
+ 		"plist_move_down",
+ 		"Move playlist item down",
+ 		CON_MENU,
+ 		{ 'j', -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_ADD_STREAM,
+ 		"plist_add_stream",
+ 		"Add a URL to the playlist using entry",
+ 		CON_MENU,
+ 		{ CTRL('U'), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_THEME_MENU,
+ 		"theme_menu",
+ 		"Switch to the theme selection menu",
+ 		CON_MENU,
+ 		{ 'T', -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC1,
+ 		"exec_command1",
+ 		"Execute ExecCommand1",
+ 		CON_MENU,
+ 		{ KEY_F(1), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC2,
+ 		"exec_command2",
+ 		"Execute ExecCommand2",
+ 		CON_MENU,
+ 		{ KEY_F(2), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC3,
+ 		"exec_command3",
+ 		"Execute ExecCommand3",
+ 		CON_MENU,
+ 		{ KEY_F(3), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC4,
+ 		"exec_command4",
+ 		"Execute ExecCommand4",
+ 		CON_MENU,
+ 		{ KEY_F(4), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC5,
+ 		"exec_command5",
+ 		"Execute ExecCommand5",
+ 		CON_MENU,
+ 		{ KEY_F(5), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC6,
+ 		"exec_command6",
+ 		"Execute ExecCommand6",
+ 		CON_MENU,
+ 		{ KEY_F(6), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC7,
+ 		"exec_command7",
+ 		"Execute ExecCommand7",
+ 		CON_MENU,
+ 		{ KEY_F(7), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC8,
+ 		"exec_command8",
+ 		"Execute ExecCommand8",
+ 		CON_MENU,
+ 		{ KEY_F(8), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC9,
+ 		"exec_command9",
+ 		"Execute ExecCommand9",
+ 		CON_MENU,
+ 		{ KEY_F(9), -1 },
+ 		1
+  	},
+ 	{
+ 		KEY_CMD_EXEC10,
+ 		"exec_command10",
+ 		"Execute ExecCommand10",
+ 		CON_MENU,
+ 		{ KEY_F(10), -1 },
+ 		1
   	}
 };
 
@@ -945,3 +1067,25 @@ char **get_keys_help (int *num)
 	*num = COMMANDS_NUM;
 	return help;
 }
+
+static int find_command (const enum key_cmd cmd)
+{
+	unsigned int i;
+
+	for (i = 0; i < COMMANDS_NUM; i++)
+		if (commands[i].cmd == cmd)
+			return i;
+
+	return -1;
+}
+
+/* Return non-zero value if the key for the command was redefined (using
+ * custom keymap). */
+int key_was_redefined (const enum key_cmd cmd)
+{
+	int i = find_command (cmd);
+
+	assert (i != -1);
+
+	return !commands[i].default_keys_set;
+}
