@@ -20,6 +20,8 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/time.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 #include <unistd.h>
 #include <signal.h>
 #define _GNU_SOURCE
@@ -27,16 +29,16 @@
 #include <errno.h>
 #include <unistd.h>
 #include <time.h>
+
 #include "server.h"
-#include "main.h"
+#include "common.h"
 #include "interface.h"
 #include "options.h"
 #include "protocol.h"
-#include "decoder.h"
 #include "log.h"
-#include "files.h"
-#include "playlist.h"
 #include "compat.h"
+#include "common.h"
+#include "decoder.h"
 
 struct parameters
 {
@@ -56,103 +58,31 @@ struct parameters
 	int previous;
 	int get_file_info;
 	int toggle_pause;
-	int recursively;
 	int playit;
+	int seek_by;
 };
 
-static int im_server = 0; /* I em the server? */
 
-void error (const char *format, ...)
+/* Connect to the server, return fd os the socket or -1 on error */
+static int server_connect ()
 {
-	va_list va;
-	char msg[256];
+	struct sockaddr_un sock_name;
+	int sock;
 	
-	va_start (va, format);
-	vsnprintf (msg, sizeof(msg), format, va);
-	msg[sizeof(msg) - 1] = 0;
-	va_end (va);
+	/* Create a socket */
+	if ((sock = socket (PF_LOCAL, SOCK_STREAM, 0)) == -1)
+		 return -1;
 	
-	if (im_server)
-		server_error (msg);
-	else
-		interface_error (msg);
-}
+	sock_name.sun_family = AF_LOCAL;
+	strcpy (sock_name.sun_path, socket_name());
 
-/* End program with a message. Use when an error occurs and we can't recover. */
-void fatal (const char *format, ...)
-{
-	va_list va;
-	char msg[256];
-	
-	va_start (va, format);
-	vsnprintf (msg, sizeof(msg), format, va);
-	msg[sizeof(msg) - 1] = 0;
-	fprintf (stderr, "\nFATAL_ERROR: %s\n\n", msg);
-	logit ("FATAL ERROR: %s", msg);
-	va_end (va);
-
-	exit (EXIT_FATAL);
-}
-
-void *xmalloc (const size_t size)
-{
-	void *p;
-
-	if ((p = malloc(size)) == NULL)
-		fatal ("Can't allocate memory!");
-	return p;
-}
-
-void *xcalloc (size_t nmemb, size_t size)
-{
-	void *p;
-
-	if ((p = calloc(nmemb, size)) == NULL)
-		fatal ("Can't allocate memory!");
-	return p;
-}
-
-void *xrealloc (void *ptr, const size_t size)
-{
-	void *p;
-
-	if ((p = realloc(ptr, size)) == NULL && size != 0)
-		fatal ("Can't allocate memory!");
-	return p;
-
-}
-
-char *xstrdup (const char *s)
-{
-	char *n;
-
-	if (s && (n = strdup(s)) == NULL)
-		fatal ("Can't allocate memory!");
-
-	return s ? n : NULL;
-}
-
-/* Return path to a file in MOC config directory. NOT THREAD SAFE */
-char *create_file_name (const char *file)
-{
-	char *home_dir;
-	static char fname[PATH_MAX];
-	char *moc_dir = options_get_str ("MOCDir");
-	
-	if (moc_dir[0] == '~') {
-		if (!(home_dir = getenv("HOME")))
-			fatal ("No HOME environmential variable.");
-		if (snprintf(fname, sizeof(fname), "%s/%s/%s", home_dir,
-				(moc_dir[1] == '/') ? moc_dir + 2 : moc_dir + 1,
-				file)
-				>= (int)sizeof(fname))
-			fatal ("Path too long.");
+	if (connect(sock, (struct sockaddr *)&sock_name,
+				SUN_LEN(&sock_name)) == -1) {
+		close (sock);
+		return -1;
 	}
-	else if (snprintf(fname, sizeof(fname), "%s/%s", moc_dir, file)
-			>= (int)sizeof(fname))
-		fatal ("Path too long.");
 
-	return fname;
+	return sock;
 }
 
 /* Ping the server. return 1 if the server respond with EV_PONG, otherwise 1. */
@@ -189,18 +119,6 @@ static void check_moc_dir ()
 	}
 }
 
-/* Return 1 if the file is a directory, 0 if not, -1 on error. */
-int isdir (const char *file)
-{
-	struct stat file_stat;
-
-	if (stat(file, &file_stat) == -1) {
-		error ("Can't stat %s: %s", file, strerror(errno));
-		return -1;
-	}
-	return S_ISDIR(file_stat.st_mode) ? 1 : 0;
-}
-
 static void sig_chld (int sig ATTR_UNUSED)
 {
 	logit ("Got SIGCHLD");
@@ -230,7 +148,7 @@ static void start_moc (const struct parameters *params, char **args,
 
 		switch (fork()) {
 			case 0: /* child - start server */
-				im_server = 1;
+				set_me_server ();
 				list_sock = server_init (params->debug,
 						params->foreground);
 				write (notify_pipe[1], &i, sizeof(i));
@@ -256,7 +174,7 @@ static void start_moc (const struct parameters *params, char **args,
 	else if (!params->foreground && params->only_server)
 		fatal ("Server is already running");
 	else if (params->foreground && params->only_server) {
-		im_server = 1;
+		set_me_server ();
 		list_sock = server_init (params->debug, params->foreground);
 		server_loop (list_sock);
 	}
@@ -268,8 +186,7 @@ static void start_moc (const struct parameters *params, char **args,
 		if (ping_server(server_sock)) {
 			if (!params->dont_run_iface) {
 				init_interface (server_sock, params->debug,
-						args, arg_num,
-						params->recursively);
+						args, arg_num);
 				interface_loop ();
 				interface_end ();
 			}
@@ -289,6 +206,9 @@ static void start_moc (const struct parameters *params, char **args,
 			interface_cmdline_play_first (server_sock);
 		if (params->get_file_info)
 			interface_cmdline_file_info (server_sock);
+		if (params->seek_by)
+			interface_cmdline_seek_by (server_sock,
+					params->seek_by);
 		send_int (server_sock, CMD_DISCONNECT);
 	}
 	else if (params->only_server)
@@ -353,11 +273,11 @@ static void show_usage (const char *prg_name) {
 "-F --foreground        Run server in foreground, log to stdout.\n"
 "-R --sound-driver NAME Use the specified sound driver (oss, alsa, jack, null).\n"
 "-m --music-dir         Start in MusicDir.\n"
-"-a --append            Append the files passed in command line to playlist\n"
-"                       and exit.\n"
+"-a --append            Append the files/directories/playlists passed in\n"
+"                       the command line to playlist and exit.\n"
 "-c --clear             Clear the playlist and exit.\n"
 "-p --play              Start playing from the first item on the playlist.\n"
-"-l --playit            Play files given on command line without modifing the"
+"-l --playit            Play files given on command line without modifing the\n"
 "                       playlist.\n"
 "-s --stop              Stop playing.\n"
 "-f --next              Play next song.\n"
@@ -374,8 +294,8 @@ static void show_usage (const char *prg_name) {
 "-n --nosync            Don't synchronize the playlist with other clients.\n"
 "-A --ascii             Use ASCII characters to draw lines.\n"
 "-i --info              Print the information about the currently played file.\n"
-"-e --recursively       Make a playlist from the content of the directory given\n"
-"                       at the command line.\n"
+"-e --recursively       Alias for -a.\n"
+"-k --seek N            Seek by N seconds (can be negative).\n"
 , prg_name);
 }
 
@@ -446,6 +366,18 @@ static void server_command (struct parameters *params)
 	close (sock);
 }
 
+static long get_num_param (const char *p)
+{
+	char *e;
+	long val;
+
+	val = strtol (p, &e, 10);
+	if (*e)
+		fatal ("The parameter should be a number.");
+
+	return val;
+}
+
 int main (int argc, char *argv[])
 {
 	struct option long_options[] = {
@@ -477,6 +409,7 @@ int main (int argc, char *argv[])
 		{ "ascii",		0, NULL, 'A' },
 		{ "info",		0, NULL, 'i' },
 		{ "recursively",	0, NULL, 'e' },
+		{ "seek",		1, NULL, 'k' },
 		{ 0, 0, 0, 0 }
 	};
 	int ret, opt_index = 0;
@@ -486,7 +419,8 @@ int main (int argc, char *argv[])
 	memset (&params, 0, sizeof(params));
 	options_init ();
 
-	while ((ret = getopt_long(argc, argv, "VhDSFR:macpsxT:C:M:PUynArfiGel",
+	while ((ret = getopt_long(argc, argv,
+					"VhDSFR:macpsxT:C:M:PUynArfiGelk:",
 					long_options, &opt_index)) != -1) {
 		switch (ret) {
 			case 'V':
@@ -517,6 +451,7 @@ int main (int argc, char *argv[])
 				option_ignore_config ("StartInMusicDir");
 				break;
 			case 'a':
+			case 'e':
 				params.append = 1;
 				params.dont_run_iface = 1;
 				break;
@@ -586,8 +521,9 @@ int main (int argc, char *argv[])
 				params.toggle_pause = 1;
 				params.dont_run_server = 1;
 				break;
-			case 'e':
-				params.recursively = 1;
+			case 'k':
+				params.seek_by = get_num_param (optarg);
+				params.dont_run_iface = 1;
 				break;
 			default:
 				show_usage (argv[0]);
